SystemOrganization addCategory: #'Math-StatisticalMoments'!

Object subclass: #DhbHistogram
	instanceVariableNames: 'minimum binWidth overflow underflow moments contents freeExtent cacheSize desiredNumberOfBins'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-StatisticalMoments'!

!DhbHistogram commentStamp: 'StephaneDucasse 7/12/2015 09:49' prior: 0!
[[[
| histogram valueStream |
	valueStream := #(3 3 5 6 7 7 7 72  1 3  5 7 8 20) readStream.
      histogram := DhbHistogram new.
      [ valueStream atEnd ]
		whileFalse: [ histogram accumulate: valueStream next ].
histogram			
]]]!

!DhbHistogram class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultCacheSize
		"Private - Answer the default cache size.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^100! !

!DhbHistogram class methodsFor: 'information' stamp: 'StephaneDucasse 7/12/2015 10:35'!
defaultNumberOfBins
	"Private - Defines the default number of bins for instances of the receiver. "
	^ 50! !

!DhbHistogram class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
integerScales
	"Private - Scales for strict integers"
	^#( 2 4 5 8 10)! !

!DhbHistogram class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Create a standard new instance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^super new initialize! !

!DhbHistogram class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
scales
	"Private - Scales for any number"
	^#( 1.25 2 2.5 4 5 7.5 8 10)! !

!DhbHistogram class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
semiIntegerScales
	"Private - Scales for large integers"
	^#( 2 2.5 4 5 7.5 8 10)! !

!DhbHistogram methodsFor: 'transformation' stamp: 'StephaneDucasse 7/12/2015 10:41'!
accumulate: aNumber
	"Accumulate aNumber into the receiver. "

	| bin |
	self isCached
		ifTrue: [ ^ self accumulateInCache: aNumber ].
	bin := self binIndex: aNumber.
	(bin between: 1 and: contents size)
		ifTrue: [ 
			contents at: bin put: (contents at: bin) + 1.
			moments accumulate: aNumber ]
		ifFalse: [ self processOverflows: bin for: aNumber ]! !

!DhbHistogram methodsFor: 'private' stamp: 'StephaneDucasse 7/12/2015 10:41'!
accumulateInCache: aNumber
	"Private - Accumulate aNumber inside a cache "

	contents add: aNumber.
	contents size > cacheSize
		ifTrue: [ self flushCache ]! !

!DhbHistogram methodsFor: 'private' stamp: 'StephaneDucasse 7/12/2015 10:41'!
adjustDimensionUpTo: aNumber
	"Private - Compute an adequate bin width and adjust the minimum and number of bins accordingly.
	aNumber is the maximum value to accumulate. The minimum value has already been assigned. "

	| maximum |
	binWidth := self roundToScale: (aNumber - minimum) / desiredNumberOfBins.
	minimum := (minimum / binWidth) floor * binWidth.
	maximum := (aNumber / binWidth) ceiling * binWidth.
	contents := Array new: ((maximum - minimum) / binWidth) ceiling.
	contents atAllPut: 0! !

!DhbHistogram methodsFor: 'information' stamp: 'StephaneDucasse 7/12/2015 09:50'!
average
	"Answer the average of the recevier
		"
	^ self moments average! !

!DhbHistogram methodsFor: 'information' stamp: 'StephaneDucasse 7/12/2015 09:51'!
binIndex: aNumber
	"Answers the index of the bin corresponding to aNumber."
	
	^ ((aNumber - minimum) / binWidth) floor + 1! !

!DhbHistogram methodsFor: 'information' stamp: 'StephaneDucasse 7/12/2015 09:52'!
binWidth
	"Answer the bin width for the receiver."

	self isCached
		ifTrue: [ self flushCache ].
	^ binWidth! !

!DhbHistogram methodsFor: 'information' stamp: 'StephaneDucasse 7/12/2015 09:52'!
chi2Against: aScaledDistribution

	| chi2 |
	chi2 := 0.
	self pointsAndErrorsDo: [ :each | chi2 := (each chi2Contribution: aScaledDistribution) + chi2 ].
	^ chi2! !

!DhbHistogram methodsFor: 'information' stamp: 'StephaneDucasse 7/12/2015 09:52'!
chi2ConfidenceLevelAgainst: aScaledDistribution

	^ (DhbChiSquareDistribution degreeOfFreedom: contents size - aScaledDistribution parameters size)
		confidenceLevel: (self chi2Against: aScaledDistribution)! !

!DhbHistogram methodsFor: 'information' stamp: 'StephaneDucasse 7/12/2015 09:53'!
collectIntegralPoints: aBlock
	"Collects the points needed to display the receiver as an integral. 
	Needed to use polymorphic behavior when plotting the receiver."

	| answer bin integral norm x |
	self isCached
		ifTrue: [ self flushCache ].
	answer := OrderedCollection new: contents size * 2 + 1.
	bin := self minimum.
	answer add: (aBlock value: bin @ 0).
	integral := self underflow.
	norm := self totalCount.
	contents
		do: [ :each | 
			integral := integral + each.
			x := integral / norm.
			answer add: (aBlock value: bin @ x).
			bin := bin + binWidth.
			answer add: (aBlock value: bin @ x) ].
	answer add: (aBlock value: bin @ 0).
	^ answer asArray! !

!DhbHistogram methodsFor: 'information' stamp: 'StephaneDucasse 7/12/2015 09:53'!
collectPoints: aBlock
	"Collects the points needed to display the receiver.
	Needed to use polymorphic behavior when plotting the receiver."

	| answer bin |
	self isCached
		ifTrue: [ self flushCache ].
	answer := OrderedCollection new: contents size * 2 + 1.
	bin := self minimum.
	answer add: (aBlock value: bin @ 0).
	contents
		do: [ :each | 
			answer add: (aBlock value: bin @ each).
			bin := bin + binWidth.
			answer add: (aBlock value: bin @ each) ].
	answer add: (aBlock value: bin @ 0).
	^ answer asArray! !

!DhbHistogram methodsFor: 'information' stamp: 'StephaneDucasse 7/12/2015 09:53'!
count
	"Answer the count of the recevier"

	^ self moments count! !

!DhbHistogram methodsFor: 'information' stamp: 'WernerKassens 4/5/2014 13:05'!
countAt: aNumber
		"Answer the count in the bin corresponding to aNumber or 0 if outside the limits.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	| n |
	self isCached 				
		ifTrue: [ self flushCache].
	n := self binIndex: aNumber.
	^( n between: 1 and: contents size)
			ifTrue: [ contents at: n]
			ifFalse:[ 0]! !

!DhbHistogram methodsFor: 'transformation' stamp: 'StephaneDucasse 7/12/2015 10:41'!
countOverflows: anInteger

	anInteger > 0
		ifTrue: [ overflow := overflow + 1 ]
		ifFalse: [ underflow := underflow + 1 ]! !

!DhbHistogram methodsFor: 'information' stamp: 'StephaneDucasse 7/12/2015 09:56'!
countsBetween: aNumber1 and: aNumber2
	"Computes the events located between aNumber1 and aNumber2.
	NOTE: This method assumes the two numbers are within the limits of the receiver."

	"there was a little bug that just seems to be one of the authors typical slips of the pen"

	| n1 n2 answer |
	self isCached
		ifTrue: [ self flushCache ].
	n1 := self binIndex: aNumber1.
	n2 := self binIndex: aNumber2.
	answer := (contents at: n1) * (binWidth * (n1 - 1) + minimum - aNumber1) / binWidth.
	n2 > contents size
		ifTrue: [ n2 := contents size + 1 ]
		ifFalse: [ answer := answer + ((contents at: n2) * (aNumber2 - (binWidth * (n2 - 1) + self minimum)) / binWidth) ].
	n1 to: n2 - 1 do: [ :n | answer := answer + (contents at: n) ].
	^ answer! !

!DhbHistogram methodsFor: 'information' stamp: 'StephaneDucasse 7/12/2015 09:56'!
countsUpTo: aNumber
	"Computes the events located up to aNumber.
	NOTE: This method assumes aNumber is within the limits of the receiver."

	| n answer |
	self isCached
		ifTrue: [ self flushCache ].
	n := self binIndex: aNumber.
	n > contents size
		ifTrue: [ ^ self count ].
	answer := (contents at: n) * (aNumber - (binWidth * (n - 1) + minimum)) / binWidth.
	1 to: n - 1 do: [ :m | answer := answer + (contents at: m) ].
	^ answer + underflow! !

!DhbHistogram methodsFor: 'information' stamp: 'StephaneDucasse 7/12/2015 09:56'!
errorOnAverage
	"Answer the error on the average of the receiver."

	^ self moments errorOnAverage! !

!DhbHistogram methodsFor: 'testing' stamp: 'StephaneDucasse 7/12/2015 10:33'!
fConfidenceLevel: aStatisticalMomentsOrHistogram

	^ self moments fConfidenceLevel: aStatisticalMomentsOrHistogram! !

!DhbHistogram methodsFor: 'private' stamp: 'StephaneDucasse 7/12/2015 10:41'!
flushCache
	"Private"

	| maximum values |
	minimum isNil
		ifTrue: [ 
			minimum := contents isEmpty
				ifTrue: [ 0 ]
				ifFalse: [ contents first ] ].
	maximum := minimum.
	contents
		do: [ :each | 
			each < minimum
				ifTrue: [ minimum := each ]
				ifFalse: [ 
					each > maximum
						ifTrue: [ maximum := each ] ] ].
	maximum = minimum
		ifTrue: [ maximum := minimum + desiredNumberOfBins ].
	values := contents.
	self adjustDimensionUpTo: maximum.
	values do: [ :each | self accumulate: each ]! !

!DhbHistogram methodsFor: 'initialization' stamp: 'StephaneDucasse 7/12/2015 10:35'!
freeExtent: aBoolean
	"Defines the range of the receiver to be freely adjustable."

	(underflow = 0 and: [ overflow = 0 ])
		ifFalse: [ self error: 'Histogram extent cannot be redefined' ].
	freeExtent := aBoolean! !

!DhbHistogram methodsFor: 'private' stamp: 'StephaneDucasse 7/12/2015 11:14'!
growContents: anInteger
	"Private "
	anInteger > 0
		ifTrue: [ self growPositiveContents: anInteger ]
		ifFalse: [ self growNegativeContents: anInteger ]! !

!DhbHistogram methodsFor: 'private' stamp: 'StephaneDucasse 7/12/2015 10:42'!
growNegativeContents: anInteger
	"Private"

	| n newSize newContents |
	n := 1 - anInteger.
	newSize := contents size + n.
	newContents := Array new: newSize.
	newContents at: 1 put: 1.
	2 to: n do: [ :i | newContents at: i put: 0 ].
	newContents
		replaceFrom: n + 1
		to: newSize
		with: contents
		startingAt: 1.
	contents := newContents.
	minimum := (anInteger - 1) * binWidth + minimum! !

!DhbHistogram methodsFor: 'private' stamp: 'StephaneDucasse 7/12/2015 10:42'!
growPositiveContents: anInteger
	"Private"

	| n newContents |
	n := contents size.
	newContents := Array new: anInteger.
	newContents
		replaceFrom: 1
		to: n
		with: contents
		startingAt: 1.
	n + 1 to: anInteger - 1 do: [ :i | newContents at: i put: 0 ].
	newContents at: anInteger put: 1.
	contents := newContents! !

!DhbHistogram methodsFor: 'initialization' stamp: 'StephaneDucasse 7/12/2015 10:35'!
initialize
	"Private - initializes the receiver with standard settings."

	freeExtent := false.
	cacheSize := self class defaultCacheSize.
	desiredNumberOfBins := self class defaultNumberOfBins.
	contents := OrderedCollection new: cacheSize.
	moments := DhbFixedStatisticalMoments new.
	overflow := 0.
	underflow := 0.
	^ self! !

!DhbHistogram methodsFor: 'information' stamp: 'StephaneDucasse 7/12/2015 09:57'!
inverseDistributionValue: aNumber
	"Private - Compute the value which corresponds to a integrated count of aNumber. NOTE: aNumber is assumed to be between 0 and 1."

	| count x integral |
	count := self count * aNumber.
	x := minimum.
	integral := 0.
	contents
		do: [ :each | 
			| delta |
			delta := count - integral.
			each > delta
				ifTrue: [ ^ binWidth * delta / each + x ].
			integral := integral + each.
			x := binWidth + x ].
	^ self maximum! !

!DhbHistogram methodsFor: 'testing' stamp: 'StephaneDucasse 7/12/2015 09:57'!
isCached
	"Private - Answer true if the content of the receiver is cached."

	^ binWidth isNil! !

!DhbHistogram methodsFor: 'testing' stamp: 'StephaneDucasse 7/12/2015 10:33'!
isEmpty
	"Always false. 
	 Needed to use polymorphic behavior when plotting the receiver. "

	^ false! !

!DhbHistogram methodsFor: 'information' stamp: 'WernerKassens 4/5/2014 12:47'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^self moments kurtosis! !

!DhbHistogram methodsFor: 'information' stamp: 'WernerKassens 4/5/2014 13:07'!
lowBinLimitAt: anInteger
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	self isCached 				
		ifTrue: [ self flushCache].
	^( anInteger - 1) * binWidth + minimum! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
maximum
		"Answer the minimum for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	self isCached
		ifTrue: [ self flushCache].
	^contents size * binWidth + minimum! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
maximumCount
		"Answer the maximum count of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	self isCached
		ifTrue: [ self flushCache].
	^contents inject: ( contents isEmpty ifTrue: [ 1] ifFalse:[ contents at: 1])
					into: [ :max :each | max max: each]! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
minimum
		"Answer the minimum for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	self isCached
		ifTrue: [ self flushCache].
	^minimum! !

!DhbHistogram methodsFor: 'information' stamp: 'WernerKassens 4/5/2014 12:44'!
moments
self isCached
	ifTrue: [ self flushCache].
^moments! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
overflow
		"Answer the overflow of the recevier
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^overflow! !

!DhbHistogram methodsFor: 'iterators' stamp: 'StephaneDucasse 7/12/2015 10:43'!
pointsAndErrorsDo: aBlock

	| x |
	x := self minimum - (binWidth / 2).
	contents
		do: [ :each | 
			x := x + binWidth.
			aBlock value: (DhbWeightedPoint point: x count: each) ]! !

!DhbHistogram methodsFor: 'private' stamp: 'StephaneDucasse 7/12/2015 10:42'!
processOverflows: anInteger for: aNumber
	"Private"

	freeExtent
		ifTrue: [ 
			self growContents: anInteger.
			moments accumulate: aNumber ]
		ifFalse: [ self countOverflows: anInteger ]! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
roundToScale: aNumber
	"Private - Adjust aNumber of the lowest upper scale"
	| orderOfMagnitude norm scales rValue |
	orderOfMagnitude := ( aNumber log: 10) floor.
	scales := self class scales.
	aNumber isInteger
		ifTrue: [ orderOfMagnitude < 1
						ifTrue: [ orderOfMagnitude := 1].
					  orderOfMagnitude = 1
						ifTrue: [ scales := self class integerScales].
					  orderOfMagnitude = 2
						ifTrue: [ scales := self class semiIntegerScales].
				    ].
	norm := 10 raisedToInteger: orderOfMagnitude.
	rValue := aNumber / norm.
	^( scales detect: [ :each | rValue <= each]) * norm! !

!DhbHistogram methodsFor: 'initialization' stamp: 'StephaneDucasse 7/12/2015 10:36'!
setDesiredNumberOfBins: anInteger
	"Defines the desired number of bins. It may be adjusted to a few units later on. "

	anInteger > 0
		ifFalse: [ self error: 'Desired number of bins must be positive' ].
	desiredNumberOfBins := anInteger! !

!DhbHistogram methodsFor: 'initialization' stamp: 'StephaneDucasse 7/12/2015 10:36'!
setRangeFrom: aNumber1 to: aNumber2 bins: anInteger
	"Defines the range of the receiver by specifying the minimum, maximum and number of bins. Values are adjusted to correspond to a reasonable value for the bin width and the limits."

	self isCached
		ifFalse: [ self error: 'Histogram limits cannot be redefined' ].
	minimum := aNumber1.
	self
		setDesiredNumberOfBins: anInteger;
		adjustDimensionUpTo: aNumber2! !

!DhbHistogram methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setWidth: aNumber1 from: aNumber2 bins: anInteger
		"Defines the range of the receiver by specifying the minimum, bin width and number of bins.
		 Values are adjusted to correspond to a reasonable value for the bin width and the limits.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	self isCached
		ifFalse: [ self error: 'Histogram limits cannot be redefined'].
	minimum := aNumber2.
	self setDesiredNumberOfBins: anInteger;
		   adjustDimensionUpTo: ( aNumber1 * anInteger + aNumber2).! !

!DhbHistogram methodsFor: 'information' stamp: 'StephaneDucasse 7/12/2015 10:34'!
skewness
	"Answer the skewness of the receiver"

	^ self moments skewness! !

!DhbHistogram methodsFor: 'information' stamp: 'StephaneDucasse 7/12/2015 10:34'!
standardDeviation
	"Answer the standardDeviation of the recevier"

	^ self moments standardDeviation! !

!DhbHistogram methodsFor: 'testing' stamp: 'StephaneDucasse 7/12/2015 10:33'!
tConfidenceLevel: aStatisticalMomentsOrHistogram

	^ self moments tConfidenceLevel: aStatisticalMomentsOrHistogram! !

!DhbHistogram methodsFor: 'information' stamp: 'StephaneDucasse 7/12/2015 10:34'!
totalCount
	"Answer the count of the recevier, inclusing underflow and overflow"

	^ self moments count + underflow + overflow! !

!DhbHistogram methodsFor: 'information' stamp: 'StephaneDucasse 7/12/2015 10:34'!
underflow
	"Answer the underflow of the receiver"

	^ underflow! !

!DhbHistogram methodsFor: 'information' stamp: 'StephaneDucasse 7/12/2015 10:34'!
unnormalizedVariance

	^ self moments unnormalizedVariance! !

!DhbHistogram methodsFor: 'information' stamp: 'StephaneDucasse 7/12/2015 10:34'!
variance
	"Answer the variance of the recevier "

	^ self moments variance! !

Object subclass: #DhbStatisticalMoments
	instanceVariableNames: 'moments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-StatisticalMoments'!

DhbStatisticalMoments subclass: #DhbFastStatisticalMoments
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-StatisticalMoments'!

!DhbFastStatisticalMoments methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulate: aNumber 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "

	| var |
	var := 1.
	1 to: moments size
		do: 
			[:n | 
			moments at: n put: (moments at: n) + var.
			var := var * aNumber]! !

!DhbFastStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "

	self count = 0 ifTrue: [^nil].
	^(moments at: 2) / self count! !

!DhbFastStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
kurtosis
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "

	| var x1 x2 x3 x4 kFact kConst n m4 xSquared |
	n := self count.
	n < 4 ifTrue: [^nil].
	var := self variance.
	var = 0 ifTrue: [^nil].
	x1 := (moments at: 2) / n.
	x2 := (moments at: 3) / n.
	x3 := (moments at: 4) / n.
	x4 := (moments at: 5) / n.
	xSquared := x1 squared.
	m4 := x4 - (4 * x1 * x3) + (6 * x2 * xSquared) - (xSquared squared * 3).
	kFact := n * (n + 1) / (n - 1) / (n - 2) / (n - 3).
	kConst := 3 * (n - 1) * (n - 1) / (n - 2) / (n - 3).
	^kFact * (m4 * n / var squared) - kConst! !

!DhbFastStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
skewness
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "

	| x1 x2 x3 n stdev |
	n := self count.
	n < 3 ifTrue: [^nil].
	stdev := self standardDeviation.
	stdev = 0 ifTrue: [^nil].
	x1 := (moments at: 2) / n.
	x2 := (moments at: 3) / n.
	x3 := (moments at: 4) / n.
	^(x3 - (3 * x1 * x2) + (2 * x1 * x1 * x1)) * n * n 
		/ (stdev squared * stdev * (n - 1) * (n - 2))! !

!DhbFastStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
unnormalizedVariance
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/22/00 "

	^(moments at: 3) - ((moments at: 2) squared * self count)! !

!DhbFastStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
variance
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "

	| n |
	n := self count.
	n < 2 ifTrue: [^nil].
	^((moments at: 3) - ((moments at: 2) squared / n)) / (n - 1)! !

DhbStatisticalMoments subclass: #DhbFixedStatisticalMoments
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-StatisticalMoments'!

!DhbFixedStatisticalMoments class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "
	^super new: 4! !

!DhbFixedStatisticalMoments class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: anInteger
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbFixedStatisticalMoments methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulate: aNumber 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "

	| correction n n1 c2 c3 |
	n := moments at: 1.
	n1 := n + 1.
	correction := ((moments at: 2) - aNumber) / n1.
	c2 := correction squared.
	c3 := c2 * correction.
	moments
		at: 5
			put: ((moments at: 5) + ((moments at: 4) * correction * 4) 
					+ ((moments at: 3) * c2 * 6) + (c2 squared * (n squared * n + 1))) 
					* n / n1;
		at: 4
			put: ((moments at: 4) + ((moments at: 3) * correction * 3) 
					+ (c3 * (1 - n squared))) * n 
					/ n1;
		at: 3 put: ((moments at: 3) + (c2 * (1 + n))) * n / n1;
		at: 2 put: (moments at: 2) - correction;
		at: 1 put: n1! !

!DhbStatisticalMoments class methodsFor: 'creation' stamp: 'StephaneDucasse 7/14/2015 14:02'!
new

	^ self new: 4! !

!DhbStatisticalMoments class methodsFor: 'creation' stamp: 'StephaneDucasse 7/14/2015 13:23'!
new: anInteger
	"anInteger is the degree of the highest central moments accumulated within the created instance."

	^ self basicNew initialize: anInteger + 1! !

!DhbStatisticalMoments methodsFor: 'transformation' stamp: 'StephaneDucasse 7/14/2015 14:03'!
accumulate: aNumber

	| correction n n1 oldSums pascal nTerm cTerm term |
	n := moments at: 1.
	n1 := n + 1.
	correction := ((moments at: 2) - aNumber) / n1.
	oldSums := moments copyFrom: 1 to: moments size.
	moments
		at: 1 put: n1;
		at: 2 put: (moments at: 2) - correction.
	pascal := Array new: moments size.
	pascal atAllPut: 0.
	pascal
		at: 1 put: 1;
		at: 2 put: 1.
	nTerm := -1.
	cTerm := correction.
	n1 := n / n1.
	n := n negated.
	3 to: moments size do: [ :k | 
		cTerm := cTerm * correction.
		nTerm := n * nTerm.
		term := cTerm * (1 + nTerm).
		k to: 3 by: -1 do: [ :l | 
			pascal at: l put: (pascal at: l - 1) + (pascal at: l).
			term := (pascal at: l) * (oldSums at: l) + term.
			oldSums at: l put: (oldSums at: l) * correction ].
		pascal at: 2 put: (pascal at: 1) + (pascal at: 2).
		moments at: k put: term * n1 ]! !

!DhbStatisticalMoments methodsFor: 'creation' stamp: 'StephaneDucasse 7/14/2015 14:03'!
asWeightedPoint: aNumber

	^ DhbWeightedPoint point: aNumber @ self average error: self errorOnAverage! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'StephaneDucasse 7/14/2015 14:03'!
average

	self count = 0
		ifTrue: [ ^ nil ].
	^ moments at: 2! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'StephaneDucasse 7/14/2015 14:03'!
centralMoment: anInteger

	^ moments at: anInteger + 1! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'StephaneDucasse 7/14/2015 14:03'!
count

	^ moments at: 1! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'StephaneDucasse 7/14/2015 14:03'!
errorOnAverage

	^ (self variance / self count) sqrt! !

!DhbStatisticalMoments methodsFor: 'testing' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fConfidenceLevel: aStatisticalMomentsOrHistogram 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/22/00 "
	| fValue |
	fValue := self variance/ aStatisticalMomentsOrHistogram variance.
	^fValue < 1
		ifTrue: [ (DhbFisherSnedecorDistribution degreeOfFreedom: aStatisticalMomentsOrHistogram count
						degreeOfFreedom: self count) 
										confidenceLevel: fValue reciprocal]
		ifFalse:[ (DhbFisherSnedecorDistribution degreeOfFreedom: self count
						degreeOfFreedom: aStatisticalMomentsOrHistogram count) 
										confidenceLevel: fValue]! !

!DhbStatisticalMoments methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: anInteger 
	"Private - ( anInteger - 1) is the degree of the highest accumulated central moment.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "

	moments := Array new: anInteger.
	self reset.
	^self! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
kurtosis
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "

	| n n1 n23 |
	n := self count.
	n < 4 ifTrue: [^nil].
	n23 := (n - 2) * (n - 3).
	n1 := n - 1.
	^((moments at: 5) * n squared * (n + 1) / (self variance squared * n1) 
		- (n1 squared * 3)) / n23! !

!DhbStatisticalMoments methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "

	moments atAllPut: 0! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
skewness
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "

	| n v |
	n := self count.
	n < 3 ifTrue: [^nil].
	v := self variance.
	^(moments at: 4) * n squared / ((n - 1) * (n - 2) * (v sqrt * v))! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
standardDeviation
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "
	^self variance sqrt! !

!DhbStatisticalMoments methodsFor: 'testing' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
tConfidenceLevel: aStatisticalMomentsOrHistogram
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/22/00 "
	| sbar dof |
	dof := self count + aStatisticalMomentsOrHistogram count - 2.
	sbar := ( ( self unnormalizedVariance + aStatisticalMomentsOrHistogram unnormalizedVariance) / dof) sqrt.
	^( DhbStudentDistribution degreeOfFreedom: dof)
		confidenceLevel: ( self average - (aStatisticalMomentsOrHistogram average))
							/ ( ( 1 / self count + ( 1 / aStatisticalMomentsOrHistogram count)) sqrt * sbar)! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
unnormalizedVariance
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/22/00 "
	^( self centralMoment: 2) * self count! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
variance
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "
	| n |
	n := self count.
	n < 2
		ifTrue: [ ^nil].
	^self unnormalizedVariance / ( n - 1)! !
